import { EventEmitter, Injectable, NgModule, Output } from '@angular/core';
import { Headers, Http, Response } from '@angular/http';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/catch';
import { Observable } from 'rxjs/Rx';
import { Router } from '@angular/router';
import { Observable as Observable$1 } from 'rxjs/Observable';
import 'rxjs/add/observable/throw';
import 'rxjs/add/observable/interval';
import 'rxjs/add/observable/timer';
import { CommonModule } from '@angular/common';
var AuthConfiguration = (function () {
    function AuthConfiguration() {
        this.stsServer = 'https://localhost:44318';
        this.redirect_url = 'https://localhost:44311';
        // The Client MUST validate that the aud (audience) Claim contains its client_id value registered at the Issuer identified by the iss (issuer) Claim as an audience.
        // The ID Token MUST be rejected if the ID Token does not list the Client as a valid audience, or if it contains additional audiences not trusted by the Client.
        this.client_id = 'angularclient';
        this.response_type = 'id_token token';
        this.scope = 'dataEventRecords securedFiles openid';
        this.post_logout_redirect_uri = 'https://localhost:44311/Unauthorized';
        this.start_checksession = false;
        this.silent_renew = true;
        this.startup_route = '/dataeventrecords/list';
        // HTTP 403
        this.forbidden_route = '/Forbidden';
        // HTTP 401
        this.unauthorized_route = '/Unauthorized';
        this.log_console_warning_active = true;
        this.log_console_debug_active = false;
        // id_token C8: The iat Claim can be used to reject tokens that were issued too far away from the current time,
        // limiting the amount of time that nonces need to be stored to prevent attacks.The acceptable range is Client specific.
        this.max_id_token_iat_offset_allowed_in_seconds = 3;
    }
    return AuthConfiguration;
}());
AuthConfiguration.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
AuthConfiguration.ctorParameters = function () { return []; };
var OidcSecurityCommon = (function () {
    /**
     * @param {?} authConfiguration
     */
    function OidcSecurityCommon(authConfiguration) {
        this.authConfiguration = authConfiguration;
        this.storage_access_token = 'authorizationData';
        this.storage_id_token = 'authorizationDataIdToken';
        this.storage_is_authorized = '_isAuthorized';
        this.storage_user_data = 'userData';
        this.storage_auth_nonce = 'authNonce';
        this.storage_auth_state_control = 'authStateControl';
        this.storage_well_known_endpoints = 'wellknownendpoints';
        this.storage = sessionStorage; //localStorage;
    }
    /**
     * @param {?} key
     * @return {?}
     */
    OidcSecurityCommon.prototype.retrieve = function (key) {
        var /** @type {?} */ item = this.storage.getItem(key);
        if (item && item !== 'undefined') {
            return JSON.parse(this.storage.getItem(key));
        }
        return;
    };
    /**
     * @param {?} key
     * @param {?} value
     * @return {?}
     */
    OidcSecurityCommon.prototype.store = function (key, value) {
        this.storage.setItem(key, JSON.stringify(value));
    };
    /**
     * @return {?}
     */
    OidcSecurityCommon.prototype.resetStorageData = function () {
        this.store(this.storage_access_token, '');
        this.store(this.storage_id_token, '');
        this.store(this.storage_is_authorized, false);
        this.store(this.storage_user_data, '');
    };
    /**
     * @return {?}
     */
    OidcSecurityCommon.prototype.getAccessToken = function () {
        return this.retrieve(this.storage_access_token);
    };
    /**
     * @param {?} message
     * @return {?}
     */
    OidcSecurityCommon.prototype.logError = function (message) {
        console.error(message);
    };
    /**
     * @param {?} message
     * @return {?}
     */
    OidcSecurityCommon.prototype.logWarning = function (message) {
        if (this.authConfiguration.log_console_warning_active) {
            console.warn(message);
        }
    };
    /**
     * @param {?} message
     * @return {?}
     */
    OidcSecurityCommon.prototype.logDebug = function (message) {
        if (this.authConfiguration.log_console_debug_active) {
            console.log(message);
        }
    };
    return OidcSecurityCommon;
}());
OidcSecurityCommon.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
OidcSecurityCommon.ctorParameters = function () { return [
    { type: AuthConfiguration, },
]; };
var OidcSecurityValidation = (function () {
    /**
     * @param {?} oidcSecurityCommon
     */
    function OidcSecurityValidation(oidcSecurityCommon) {
        this.oidcSecurityCommon = oidcSecurityCommon;
    }
    /**
     * @param {?} token
     * @param {?=} offsetSeconds
     * @return {?}
     */
    OidcSecurityValidation.prototype.isTokenExpired = function (token, offsetSeconds) {
        var /** @type {?} */ decoded;
        decoded = this.getPayloadFromToken(token, false);
        return !(this.validate_id_token_exp_not_expired(decoded, offsetSeconds));
    };
    /**
     * @param {?} decoded_id_token
     * @param {?=} offsetSeconds
     * @return {?}
     */
    OidcSecurityValidation.prototype.validate_id_token_exp_not_expired = function (decoded_id_token, offsetSeconds) {
        var /** @type {?} */ tokenExpirationDate = this.getTokenExpirationDate(decoded_id_token);
        offsetSeconds = offsetSeconds || 0;
        if (tokenExpirationDate == null) {
            return false;
        }
        // Token not expired?
        return (tokenExpirationDate.valueOf() > (new Date().valueOf() + (offsetSeconds * 1000)));
    };
    /**
     * @param {?} dataIdToken
     * @return {?}
     */
    OidcSecurityValidation.prototype.validate_required_id_token = function (dataIdToken) {
        var /** @type {?} */ validated = true;
        if (!dataIdToken.hasOwnProperty('iss')) {
            validated = false;
            this.oidcSecurityCommon.logWarning('iss missing, validatation REQUIRED prorpeties in id_token');
        }
        if (!dataIdToken.hasOwnProperty('sub')) {
            validated = false;
            this.oidcSecurityCommon.logWarning('sub missing, validatation REQUIRED prorpeties in id_token');
        }
        if (!dataIdToken.hasOwnProperty('aud')) {
            validated = false;
            this.oidcSecurityCommon.logWarning('aud missing, validatation REQUIRED prorpeties in id_token');
        }
        if (!dataIdToken.hasOwnProperty('exp')) {
            validated = false;
            this.oidcSecurityCommon.logWarning('exp missing, validatation REQUIRED prorpeties in id_token');
        }
        if (!dataIdToken.hasOwnProperty('iat')) {
            validated = false;
            this.oidcSecurityCommon.logWarning('iat missing, validatation REQUIRED prorpeties in id_token');
        }
        return validated;
    };
    /**
     * @param {?} dataIdToken
     * @param {?} max_offset_allowed_in_seconds
     * @return {?}
     */
    OidcSecurityValidation.prototype.validate_id_token_iat_max_offset = function (dataIdToken, max_offset_allowed_in_seconds) {
        if (!dataIdToken.hasOwnProperty('iat')) {
            return false;
        }
        var /** @type {?} */ dateTime_iat_id_token = new Date(0); // The 0 here is the key, which sets the date to the epoch
        dateTime_iat_id_token.setUTCSeconds(dataIdToken.iat);
        max_offset_allowed_in_seconds = max_offset_allowed_in_seconds || 0;
        if (dateTime_iat_id_token == null) {
            return false;
        }
        this.oidcSecurityCommon.logDebug('validate_id_token_iat_max_offset: ' + (new Date().valueOf() - dateTime_iat_id_token.valueOf()) + ' < ' + (max_offset_allowed_in_seconds * 1000));
        return ((new Date().valueOf() - dateTime_iat_id_token.valueOf()) < (max_offset_allowed_in_seconds * 1000));
    };
    /**
     * @param {?} dataIdToken
     * @param {?} local_nonce
     * @return {?}
     */
    OidcSecurityValidation.prototype.validate_id_token_nonce = function (dataIdToken, local_nonce) {
        if (dataIdToken.nonce !== local_nonce) {
            this.oidcSecurityCommon.logDebug('Validate_id_token_nonce failed, dataIdToken.nonce: ' + dataIdToken.nonce + ' local_nonce:' + local_nonce);
            return false;
        }
        return true;
    };
    /**
     * @param {?} dataIdToken
     * @param {?} client_id
     * @return {?}
     */
    OidcSecurityValidation.prototype.validate_id_token_iss = function (dataIdToken, client_id) {
        if (dataIdToken.iss !== client_id) {
            this.oidcSecurityCommon.logDebug('Validate_id_token_iss failed, dataIdToken.iss: ' + dataIdToken.iss + ' client_id:' + client_id);
            return false;
        }
        return true;
    };
    /**
     * @param {?} dataIdToken
     * @param {?} aud
     * @return {?}
     */
    OidcSecurityValidation.prototype.validate_id_token_aud = function (dataIdToken, aud) {
        if (dataIdToken.aud !== aud) {
            this.oidcSecurityCommon.logDebug('Validate_id_token_aud failed, dataIdToken.aud: ' + dataIdToken.aud + ' client_id:' + aud);
            return false;
        }
        return true;
    };
    /**
     * @param {?} state
     * @param {?} local_state
     * @return {?}
     */
    OidcSecurityValidation.prototype.validateStateFromHashCallback = function (state, local_state) {
        if (state !== local_state) {
            this.oidcSecurityCommon.logDebug('ValidateStateFromHashCallback failed, state: ' + state + ' local_state:' + local_state);
            return false;
        }
        return true;
    };
    /**
     * @param {?} id_token_sub
     * @param {?} userdata_sub
     * @return {?}
     */
    OidcSecurityValidation.prototype.validate_userdata_sub_id_token = function (id_token_sub, userdata_sub) {
        if (id_token_sub !== userdata_sub) {
            this.oidcSecurityCommon.logDebug('validate_userdata_sub_id_token failed, id_token_sub: ' + id_token_sub + ' userdata_sub:' + userdata_sub);
            return false;
        }
        return true;
    };
    /**
     * @param {?} token
     * @param {?} encode
     * @return {?}
     */
    OidcSecurityValidation.prototype.getPayloadFromToken = function (token, encode) {
        var /** @type {?} */ data = {};
        if (typeof token !== 'undefined') {
            var /** @type {?} */ encoded = token.split('.')[1];
            if (encode) {
                return encoded;
            }
            data = JSON.parse(this.urlBase64Decode(encoded));
        }
        return data;
    };
    /**
     * @param {?} token
     * @param {?} encode
     * @return {?}
     */
    OidcSecurityValidation.prototype.getHeaderFromToken = function (token, encode) {
        var /** @type {?} */ data = {};
        if (typeof token !== 'undefined') {
            var /** @type {?} */ encoded = token.split('.')[0];
            if (encode) {
                return encoded;
            }
            data = JSON.parse(this.urlBase64Decode(encoded));
        }
        return data;
    };
    /**
     * @param {?} token
     * @param {?} encode
     * @return {?}
     */
    OidcSecurityValidation.prototype.getSignatureFromToken = function (token, encode) {
        var /** @type {?} */ data = {};
        if (typeof token !== 'undefined') {
            var /** @type {?} */ encoded = token.split('.')[2];
            if (encode) {
                return encoded;
            }
            data = JSON.parse(this.urlBase64Decode(encoded));
        }
        return data;
    };
    /**
     * @param {?} id_token
     * @param {?} jwtkeys
     * @return {?}
     */
    OidcSecurityValidation.prototype.validate_signature_id_token = function (id_token, jwtkeys) {
        if (!jwtkeys || !jwtkeys.keys) {
            return false;
        }
        var /** @type {?} */ header_data = this.getHeaderFromToken(id_token, false);
        var /** @type {?} */ kid = header_data.kid;
        var /** @type {?} */ alg = header_data.alg;
        if ('RS256' != alg) {
            this.oidcSecurityCommon.logWarning('Only RS256 supported');
            return false;
        }
        var /** @type {?} */ isValid = false;
        for (var _i = 0, _a = jwtkeys.keys; _i < _a.length; _i++) {
            var key = _a[_i];
            if (key.kid === kid) {
                var /** @type {?} */ publickey = KEYUTIL.getKey(key);
                isValid = KJUR.jws.JWS.verify(id_token, publickey, ['RS256']);
                return isValid;
            }
        }
        return isValid;
    };
    /**
     * @param {?} access_token
     * @param {?} at_hash
     * @return {?}
     */
    OidcSecurityValidation.prototype.validate_id_token_at_hash = function (access_token, at_hash) {
        var /** @type {?} */ hash = KJUR.crypto.Util.hashString(access_token, 'sha256');
        var /** @type {?} */ first128bits = hash.substr(0, hash.length / 2);
        var /** @type {?} */ testdata = hextob64u(first128bits);
        if (testdata === at_hash) {
            return true; // isValid;
        }
        return false;
    };
    /**
     * @param {?} dataIdToken
     * @return {?}
     */
    OidcSecurityValidation.prototype.getTokenExpirationDate = function (dataIdToken) {
        if (!dataIdToken.hasOwnProperty('exp')) {
            return new Date();
        }
        var /** @type {?} */ date = new Date(0); // The 0 here is the key, which sets the date to the epoch
        date.setUTCSeconds(dataIdToken.exp);
        return date;
    };
    /**
     * @param {?} str
     * @return {?}
     */
    OidcSecurityValidation.prototype.urlBase64Decode = function (str) {
        var /** @type {?} */ output = str.replace('-', '+').replace('_', '/');
        switch (output.length % 4) {
            case 0:
                break;
            case 2:
                output += '==';
                break;
            case 3:
                output += '=';
                break;
            default:
                throw 'Illegal base64url string!';
        }
        return window.atob(output);
    };
    return OidcSecurityValidation;
}());
OidcSecurityValidation.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
OidcSecurityValidation.ctorParameters = function () { return [
    { type: OidcSecurityCommon, },
]; };
var AuthWellKnownEndpoints = (function () {
    /**
     * @param {?} http
     * @param {?} authConfiguration
     * @param {?} oidcSecurityCommon
     */
    function AuthWellKnownEndpoints(http, authConfiguration, oidcSecurityCommon) {
        var _this = this;
        this.http = http;
        this.authConfiguration = authConfiguration;
        this.oidcSecurityCommon = oidcSecurityCommon;
        this.getWellKnownEndpoints = function () {
            var /** @type {?} */ headers = new Headers();
            headers.append('Content-Type', 'application/json');
            headers.append('Accept', 'application/json');
            return _this.http.get(_this.authConfiguration.stsServer + '/.well-known/openid-configuration', {
                headers: headers,
                body: ''
            }).map(function (res) { return res.json(); });
        };
        var data = this.oidcSecurityCommon.retrieve(this.oidcSecurityCommon.storage_well_known_endpoints);
        this.oidcSecurityCommon.logDebug(data);
        if (data && data !== '') {
            this.oidcSecurityCommon.logDebug('AuthWellKnownEndpoints already defined');
            this.issuer = data.issuer;
            this.jwks_uri = data.jwks_uri;
            this.authorization_endpoint = data.authorization_endpoint;
            this.token_endpoint = data.token_endpoint;
            this.userinfo_endpoint = data.userinfo_endpoint;
            this.end_session_endpoint = data.end_session_endpoint;
            if (data.check_session_iframe) {
                this.check_session_iframe = data.check_session_iframe;
            }
            if (data.revocation_endpoint) {
                this.revocation_endpoint = data.revocation_endpoint;
            }
            if (data.introspection_endpoint) {
                this.introspection_endpoint = data.introspection_endpoint;
            }
        }
        else {
            this.oidcSecurityCommon.logDebug('AuthWellKnownEndpoints first time, get from the server');
            this.getWellKnownEndpoints()
                .subscribe(function (data) {
                _this.issuer = data.issuer;
                _this.jwks_uri = data.jwks_uri;
                _this.authorization_endpoint = data.authorization_endpoint;
                _this.token_endpoint = data.token_endpoint;
                _this.userinfo_endpoint = data.userinfo_endpoint;
                _this.end_session_endpoint = data.end_session_endpoint;
                if (data.check_session_iframe) {
                    _this.check_session_iframe = data.check_session_iframe;
                }
                if (data.revocation_endpoint) {
                    _this.revocation_endpoint = data.revocation_endpoint;
                }
                if (data.introspection_endpoint) {
                    _this.introspection_endpoint = data.introspection_endpoint;
                }
                _this.oidcSecurityCommon.store(_this.oidcSecurityCommon.storage_well_known_endpoints, data);
                _this.oidcSecurityCommon.logDebug(data);
            });
        }
    }
    return AuthWellKnownEndpoints;
}());
AuthWellKnownEndpoints.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
AuthWellKnownEndpoints.ctorParameters = function () { return [
    { type: Http, },
    { type: AuthConfiguration, },
    { type: OidcSecurityCommon, },
]; };
var OidcSecurityCheckSession = (function () {
    /**
     * @param {?} authConfiguration
     * @param {?} oidcSecurityCommon
     * @param {?} authWellKnownEndpoints
     */
    function OidcSecurityCheckSession(authConfiguration, oidcSecurityCommon, authWellKnownEndpoints) {
        this.authConfiguration = authConfiguration;
        this.oidcSecurityCommon = oidcSecurityCommon;
        this.authWellKnownEndpoints = authWellKnownEndpoints;
        this.onCheckSessionChanged = new EventEmitter(true);
    }
    /**
     * @return {?}
     */
    OidcSecurityCheckSession.prototype.init = function () {
        var _this = this;
        this.sessionIframe = window.document.createElement('iframe');
        this.oidcSecurityCommon.logDebug(this.sessionIframe);
        this.sessionIframe.style.display = 'none';
        this.sessionIframe.src = this.authWellKnownEndpoints.check_session_iframe;
        window.document.body.appendChild(this.sessionIframe);
        this.iframeMessageEvent = this.messageHandler.bind(this);
        window.addEventListener('message', this.iframeMessageEvent, false);
        return Observable$1.create(function (observer) {
            _this.sessionIframe.onload = function () {
                observer.next(_this);
                observer.complete();
            };
        });
    };
    /**
     * @param {?} session_state
     * @param {?} clientId
     * @return {?}
     */
    OidcSecurityCheckSession.prototype.pollServerSession = function (session_state, clientId) {
        var _this = this;
        var /** @type {?} */ source = Observable$1.timer(3000, 3000)
            .timeInterval()
            .pluck('interval')
            .take(10000);
        var /** @type {?} */ subscription = source.subscribe(function () {
            _this.oidcSecurityCommon.logDebug(_this.sessionIframe);
            _this.sessionIframe.contentWindow.postMessage(clientId + ' ' + session_state, _this.authConfiguration.stsServer);
        }, function (err) {
            _this.oidcSecurityCommon.logError('pollServerSession error: ' + err);
        }, function () {
            _this.oidcSecurityCommon.logDebug('checksession pollServerSession completed');
        });
    };
    /**
     * @param {?} e
     * @return {?}
     */
    OidcSecurityCheckSession.prototype.messageHandler = function (e) {
        if (e.origin === this.authConfiguration.stsServer &&
            e.source === this.sessionIframe.contentWindow) {
            if (e.data === 'error') {
                this.oidcSecurityCommon.logWarning('error from checksession messageHandler');
            }
            else if (e.data === 'changed') {
                this.onCheckSessionChanged.emit();
            }
            else {
                this.oidcSecurityCommon.logDebug(e.data + ' from checksession messageHandler');
            }
        }
    };
    return OidcSecurityCheckSession;
}());
OidcSecurityCheckSession.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
OidcSecurityCheckSession.ctorParameters = function () { return [
    { type: AuthConfiguration, },
    { type: OidcSecurityCommon, },
    { type: AuthWellKnownEndpoints, },
]; };
OidcSecurityCheckSession.propDecorators = {
    'onCheckSessionChanged': [{ type: Output },],
};
var OidcSecuritySilentRenew = (function () {
    /**
     * @param {?} oidcSecurityCommon
     */
    function OidcSecuritySilentRenew(oidcSecurityCommon) {
        this.oidcSecurityCommon = oidcSecurityCommon;
    }
    /**
     * @return {?}
     */
    OidcSecuritySilentRenew.prototype.initRenew = function () {
        this.sessionIframe = window.document.createElement('iframe');
        this.oidcSecurityCommon.logDebug(this.sessionIframe);
        this.sessionIframe.style.display = 'none';
        window.document.body.appendChild(this.sessionIframe);
    };
    /**
     * @param {?} url
     * @return {?}
     */
    OidcSecuritySilentRenew.prototype.startRenew = function (url) {
        var _this = this;
        this.oidcSecurityCommon.logDebug('startRenew for URL:' + url);
        this.sessionIframe.src = url;
        return Observable$1.create(function (observer) {
            _this.sessionIframe.onload = function () {
                observer.next(_this);
                observer.complete();
            };
        });
    };
    return OidcSecuritySilentRenew;
}());
OidcSecuritySilentRenew.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
OidcSecuritySilentRenew.ctorParameters = function () { return [
    { type: OidcSecurityCommon, },
]; };
var OidcSecurityUserService = (function () {
    /**
     * @param {?} http
     * @param {?} authConfiguration
     * @param {?} oidcSecurityCommon
     * @param {?} authWellKnownEndpoints
     */
    function OidcSecurityUserService(http, authConfiguration, oidcSecurityCommon, authWellKnownEndpoints) {
        var _this = this;
        this.http = http;
        this.authConfiguration = authConfiguration;
        this.oidcSecurityCommon = oidcSecurityCommon;
        this.authWellKnownEndpoints = authWellKnownEndpoints;
        this.getIdentityUserData = function () {
            var /** @type {?} */ headers = new Headers();
            headers.append('Content-Type', 'application/json');
            headers.append('Accept', 'application/json');
            var /** @type {?} */ token = _this.oidcSecurityCommon.getAccessToken();
            if (token !== '') {
                headers.append('Authorization', 'Bearer ' + token);
            }
            return _this.http.get(_this.authWellKnownEndpoints.userinfo_endpoint, {
                headers: headers,
                body: ''
            }).map(function (res) { return res.json(); });
        };
        if (this.oidcSecurityCommon.retrieve(this.oidcSecurityCommon.storage_user_data) !== '') {
            this.userData = this.oidcSecurityCommon.retrieve(this.oidcSecurityCommon.storage_user_data);
        }
    }
    /**
     * @return {?}
     */
    OidcSecurityUserService.prototype.initUserData = function () {
        var _this = this;
        return this.getIdentityUserData()
            .map(function (data) { return _this.userData = data; });
    };
    /**
     * @param {?} error
     * @return {?}
     */
    OidcSecurityUserService.prototype.handleError = function (error) {
        this.oidcSecurityCommon.logError(error);
    };
    return OidcSecurityUserService;
}());
OidcSecurityUserService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
OidcSecurityUserService.ctorParameters = function () { return [
    { type: Http, },
    { type: AuthConfiguration, },
    { type: OidcSecurityCommon, },
    { type: AuthWellKnownEndpoints, },
]; };
var OidcSecurityService = (function () {
    /**
     * @param {?} http
     * @param {?} authConfiguration
     * @param {?} router
     * @param {?} oidcSecurityCheckSession
     * @param {?} oidcSecuritySilentRenew
     * @param {?} oidcSecurityUserService
     * @param {?} oidcSecurityCommon
     * @param {?} authWellKnownEndpoints
     */
    function OidcSecurityService(http, authConfiguration, router, oidcSecurityCheckSession, oidcSecuritySilentRenew, oidcSecurityUserService, oidcSecurityCommon, authWellKnownEndpoints) {
        var _this = this;
        this.http = http;
        this.authConfiguration = authConfiguration;
        this.router = router;
        this.oidcSecurityCheckSession = oidcSecurityCheckSession;
        this.oidcSecuritySilentRenew = oidcSecuritySilentRenew;
        this.oidcSecurityUserService = oidcSecurityUserService;
        this.oidcSecurityCommon = oidcSecurityCommon;
        this.authWellKnownEndpoints = authWellKnownEndpoints;
        this.onUserDataLoaded = new EventEmitter(true);
        this.oidcSecurityValidation = new OidcSecurityValidation(this.oidcSecurityCommon);
        this.headers = new Headers();
        this.headers.append('Content-Type', 'application/json');
        this.headers.append('Accept', 'application/json');
        if (this.oidcSecurityCommon.retrieve(this.oidcSecurityCommon.storage_is_authorized) !== '') {
            this.isAuthorized = this.oidcSecurityCommon.retrieve(this.oidcSecurityCommon.storage_is_authorized);
        }
        this.oidcSecurityCheckSession.onCheckSessionChanged.subscribe(function () { _this.onCheckSessionChanged(); });
    }
    /**
     * @return {?}
     */
    OidcSecurityService.prototype.getToken = function () {
        return this.oidcSecurityCommon.getAccessToken();
    };
    /**
     * @return {?}
     */
    OidcSecurityService.prototype.getUserData = function () {
        if (!this.isAuthorized) {
            this.oidcSecurityCommon.logError('User must be logged in before you can get the user data!');
        }
        return this.oidcSecurityUserService.userData;
    };
    /**
     * @return {?}
     */
    OidcSecurityService.prototype.authorize = function () {
        this.resetAuthorizationData();
        this.oidcSecurityCommon.logDebug('BEGIN Authorize, no auth data');
        var /** @type {?} */ nonce = 'N' + Math.random() + '' + Date.now();
        var /** @type {?} */ state = Date.now() + '' + Math.random();
        this.oidcSecurityCommon.store(this.oidcSecurityCommon.storage_auth_state_control, state);
        this.oidcSecurityCommon.store(this.oidcSecurityCommon.storage_auth_nonce, nonce);
        this.oidcSecurityCommon.logDebug('AuthorizedController created. local state: ' + this.oidcSecurityCommon.retrieve(this.oidcSecurityCommon.storage_auth_state_control));
        var /** @type {?} */ url = this.createAuthorizeUrl(nonce, state);
        window.location.href = url;
    };
    /**
     * @return {?}
     */
    OidcSecurityService.prototype.authorizedCallback = function () {
        var _this = this;
        this.oidcSecurityCommon.logDebug('BEGIN authorizedCallback, no auth data');
        this.resetAuthorizationData();
        var /** @type {?} */ hash = window.location.hash.substr(1);
        var /** @type {?} */ result = hash.split('&').reduce(function (result, item) {
            var /** @type {?} */ parts = item.split('=');
            result[parts[0]] = parts[1];
            return result;
        }, {});
        this.oidcSecurityCommon.logDebug(result);
        this.oidcSecurityCommon.logDebug('authorizedCallback created, begin token validation');
        var /** @type {?} */ access_token = '';
        var /** @type {?} */ id_token = '';
        var /** @type {?} */ authResponseIsValid = false;
        var /** @type {?} */ decoded_id_token;
        this.getSigningKeys()
            .subscribe(function (jwtKeys) {
            _this.jwtKeys = jwtKeys;
            if (!result.error) {
                // validate state
                if (_this.oidcSecurityValidation.validateStateFromHashCallback(result.state, _this.oidcSecurityCommon.retrieve(_this.oidcSecurityCommon.storage_auth_state_control))) {
                    if (_this.authConfiguration.response_type === 'id_token token') {
                        access_token = result.access_token;
                    }
                    id_token = result.id_token;
                    var /** @type {?} */ headerDecoded = void 0;
                    decoded_id_token = _this.oidcSecurityValidation.getPayloadFromToken(id_token, false);
                    headerDecoded = _this.oidcSecurityValidation.getHeaderFromToken(id_token, false);
                    // validate jwt signature
                    if (_this.oidcSecurityValidation.validate_signature_id_token(id_token, _this.jwtKeys)) {
                        // validate nonce
                        if (_this.oidcSecurityValidation.validate_id_token_nonce(decoded_id_token, _this.oidcSecurityCommon.retrieve(_this.oidcSecurityCommon.storage_auth_nonce))) {
                            // validate required fields id_token
                            if (_this.oidcSecurityValidation.validate_required_id_token(decoded_id_token)) {
                                // validate max offset from the id_token issue to now
                                if (_this.oidcSecurityValidation.validate_id_token_iat_max_offset(decoded_id_token, _this.authConfiguration.max_id_token_iat_offset_allowed_in_seconds)) {
                                    // validate iss
                                    if (_this.oidcSecurityValidation.validate_id_token_iss(decoded_id_token, _this.authWellKnownEndpoints.issuer)) {
                                        // validate aud
                                        if (_this.oidcSecurityValidation.validate_id_token_aud(decoded_id_token, _this.authConfiguration.client_id)) {
                                            // validate_id_token_exp_not_expired
                                            if (_this.oidcSecurityValidation.validate_id_token_exp_not_expired(decoded_id_token)) {
                                                // flow id_token token
                                                if (_this.authConfiguration.response_type === 'id_token token') {
                                                    // valiadate at_hash and access_token
                                                    if (_this.oidcSecurityValidation.validate_id_token_at_hash(access_token, decoded_id_token.at_hash) || !access_token) {
                                                        authResponseIsValid = true;
                                                        _this.successful_validation();
                                                    }
                                                    else {
                                                        _this.oidcSecurityCommon.logWarning('authorizedCallback incorrect at_hash');
                                                    }
                                                }
                                                else {
                                                    authResponseIsValid = true;
                                                    _this.successful_validation();
                                                }
                                            }
                                            else {
                                                _this.oidcSecurityCommon.logWarning('authorizedCallback token expired');
                                            }
                                        }
                                        else {
                                            _this.oidcSecurityCommon.logWarning('authorizedCallback incorrect aud');
                                        }
                                    }
                                    else {
                                        _this.oidcSecurityCommon.logWarning('authorizedCallback incorrect iss');
                                    }
                                }
                                else {
                                    _this.oidcSecurityCommon.logWarning('authorizedCallback Validation, iat rejected id_token was issued too far away from the current time');
                                }
                            }
                            else {
                                _this.oidcSecurityCommon.logDebug('authorizedCallback Validation, one of the REQUIRED properties missing from id_token');
                            }
                        }
                        else {
                            _this.oidcSecurityCommon.logWarning('authorizedCallback incorrect nonce');
                        }
                    }
                    else {
                        _this.oidcSecurityCommon.logWarning('authorizedCallback incorrect Signature id_token');
                    }
                }
                else {
                    _this.oidcSecurityCommon.logWarning('authorizedCallback incorrect state');
                }
            }
            if (authResponseIsValid) {
                _this.setAuthorizationData(access_token, id_token);
                // flow id_token token
                if (_this.authConfiguration.response_type === 'id_token token') {
                    _this.oidcSecurityUserService.initUserData()
                        .subscribe(function () {
                        _this.oidcSecurityCommon.logDebug('authorizedCallback id_token token flow');
                        if (_this.oidcSecurityValidation.validate_userdata_sub_id_token(decoded_id_token.sub, _this.oidcSecurityUserService.userData.sub)) {
                            _this.onUserDataLoaded.emit();
                            _this.oidcSecurityCommon.logDebug(_this.oidcSecurityCommon.retrieve(_this.oidcSecurityCommon.storage_access_token));
                            _this.oidcSecurityCommon.logDebug(_this.oidcSecurityUserService.userData);
                            if (_this.authConfiguration.start_checksession) {
                                _this.oidcSecurityCheckSession.init().subscribe(function () {
                                    _this.oidcSecurityCheckSession.pollServerSession(result.session_state, _this.authConfiguration.client_id);
                                });
                            }
                            if (_this.authConfiguration.silent_renew) {
                                _this.oidcSecuritySilentRenew.initRenew();
                            }
                            _this.runTokenValidatation();
                            _this.router.navigate([_this.authConfiguration.startup_route]);
                        }
                        else {
                            _this.oidcSecurityCommon.logWarning('authorizedCallback, User data sub does not match sub in id_token');
                            _this.oidcSecurityCommon.logDebug('authorizedCallback, token(s) validation failed, resetting');
                            _this.resetAuthorizationData();
                            _this.router.navigate([_this.authConfiguration.unauthorized_route]);
                        }
                    });
                }
                else {
                    _this.oidcSecurityCommon.logDebug('authorizedCallback id_token flow');
                    _this.oidcSecurityCommon.logDebug(_this.oidcSecurityCommon.retrieve(_this.oidcSecurityCommon.storage_access_token));
                    // userData is set to the id_token decoded. No access_token.
                    _this.oidcSecurityUserService.userData = decoded_id_token;
                    if (_this.authConfiguration.start_checksession) {
                        _this.oidcSecurityCheckSession.init().subscribe(function () {
                            _this.oidcSecurityCheckSession.pollServerSession(result.session_state, _this.authConfiguration.client_id);
                        });
                    }
                    if (_this.authConfiguration.silent_renew) {
                        _this.oidcSecuritySilentRenew.initRenew();
                    }
                    _this.runTokenValidatation();
                    _this.router.navigate([_this.authConfiguration.startup_route]);
                }
            }
            else {
                _this.oidcSecurityCommon.logDebug('authorizedCallback, token(s) validation failed, resetting');
                _this.resetAuthorizationData();
                _this.router.navigate([_this.authConfiguration.unauthorized_route]);
            }
        });
    };
    /**
     * @return {?}
     */
    OidcSecurityService.prototype.logoff = function () {
        // /connect/endsession?id_token_hint=...&post_logout_redirect_uri=https://myapp.com
        this.oidcSecurityCommon.logDebug('BEGIN Authorize, no auth data');
        var /** @type {?} */ authorizationEndsessionUrl = this.authWellKnownEndpoints.end_session_endpoint;
        var /** @type {?} */ id_token_hint = this.oidcSecurityCommon.retrieve(this.oidcSecurityCommon.storage_id_token);
        var /** @type {?} */ post_logout_redirect_uri = this.authConfiguration.post_logout_redirect_uri;
        var /** @type {?} */ url = authorizationEndsessionUrl + '?' +
            'id_token_hint=' + encodeURI(id_token_hint) + '&' +
            'post_logout_redirect_uri=' + encodeURI(post_logout_redirect_uri);
        this.resetAuthorizationData();
        if (this.authConfiguration.start_checksession && this.checkSessionChanged) {
            this.oidcSecurityCommon.logDebug('only local login cleaned up, server session has changed');
        }
        else {
            window.location.href = url;
        }
    };
    /**
     * @return {?}
     */
    OidcSecurityService.prototype.successful_validation = function () {
        this.oidcSecurityCommon.store(this.oidcSecurityCommon.storage_auth_nonce, '');
        this.oidcSecurityCommon.store(this.oidcSecurityCommon.storage_auth_state_control, '');
        this.oidcSecurityCommon.logDebug('AuthorizedCallback token(s) validated, continue');
    };
    /**
     * @return {?}
     */
    OidcSecurityService.prototype.refreshSession = function () {
        this.oidcSecurityCommon.logDebug('BEGIN refresh session Authorize');
        var /** @type {?} */ nonce = 'N' + Math.random() + '' + Date.now();
        var /** @type {?} */ state = Date.now() + '' + Math.random();
        this.oidcSecurityCommon.store(this.oidcSecurityCommon.storage_auth_state_control, state);
        this.oidcSecurityCommon.store(this.oidcSecurityCommon.storage_auth_nonce, nonce);
        this.oidcSecurityCommon.logDebug('RefreshSession created. adding myautostate: ' + this.oidcSecurityCommon.retrieve(this.oidcSecurityCommon.storage_auth_state_control));
        var /** @type {?} */ url = this.createAuthorizeUrl(nonce, state);
        this.oidcSecuritySilentRenew.startRenew(url);
    };
    /**
     * @param {?} access_token
     * @param {?} id_token
     * @return {?}
     */
    OidcSecurityService.prototype.setAuthorizationData = function (access_token, id_token) {
        if (this.oidcSecurityCommon.retrieve(this.oidcSecurityCommon.storage_access_token) !== '') {
            this.oidcSecurityCommon.store(this.oidcSecurityCommon.storage_access_token, '');
        }
        this.oidcSecurityCommon.logDebug(access_token);
        this.oidcSecurityCommon.logDebug(id_token);
        this.oidcSecurityCommon.logDebug('storing to storage, getting the roles');
        this.oidcSecurityCommon.store(this.oidcSecurityCommon.storage_access_token, access_token);
        this.oidcSecurityCommon.store(this.oidcSecurityCommon.storage_id_token, id_token);
        this.isAuthorized = true;
        this.oidcSecurityCommon.store(this.oidcSecurityCommon.storage_is_authorized, true);
    };
    /**
     * @param {?} nonce
     * @param {?} state
     * @return {?}
     */
    OidcSecurityService.prototype.createAuthorizeUrl = function (nonce, state) {
        var /** @type {?} */ authorizationUrl = this.authWellKnownEndpoints.authorization_endpoint;
        var /** @type {?} */ client_id = this.authConfiguration.client_id;
        var /** @type {?} */ redirect_uri = this.authConfiguration.redirect_url;
        var /** @type {?} */ response_type = this.authConfiguration.response_type;
        var /** @type {?} */ scope = this.authConfiguration.scope;
        var /** @type {?} */ url = authorizationUrl + '?' +
            'response_type=' + encodeURI(response_type) + '&' +
            'client_id=' + encodeURI(client_id) + '&' +
            'redirect_uri=' + encodeURI(redirect_uri) + '&' +
            'scope=' + encodeURI(scope) + '&' +
            'nonce=' + encodeURI(nonce) + '&' +
            'state=' + encodeURI(state);
        return url;
    };
    /**
     * @return {?}
     */
    OidcSecurityService.prototype.resetAuthorizationData = function () {
        this.isAuthorized = false;
        this.oidcSecurityCommon.resetStorageData();
        this.checkSessionChanged = false;
    };
    /**
     * @param {?} error
     * @return {?}
     */
    OidcSecurityService.prototype.handleError = function (error) {
        this.oidcSecurityCommon.logError(error);
        if (error.status == 403) {
            this.router.navigate([this.authConfiguration.forbidden_route]);
        }
        else if (error.status == 401) {
            this.resetAuthorizationData();
            this.router.navigate([this.authConfiguration.unauthorized_route]);
        }
    };
    /**
     * @return {?}
     */
    OidcSecurityService.prototype.onCheckSessionChanged = function () {
        this.oidcSecurityCommon.logDebug('onCheckSessionChanged');
        this.checkSessionChanged = true;
    };
    /**
     * @return {?}
     */
    OidcSecurityService.prototype.runGetSigningKeys = function () {
        var _this = this;
        this.getSigningKeys()
            .subscribe(function (jwtKeys) { return _this.jwtKeys = jwtKeys; }, function (error) { return _this.errorMessage = (error); });
    };
    /**
     * @return {?}
     */
    OidcSecurityService.prototype.getSigningKeys = function () {
        this.oidcSecurityCommon.logDebug('jwks_uri: ' + this.authWellKnownEndpoints.jwks_uri);
        return this.http.get(this.authWellKnownEndpoints.jwks_uri)
            .map(this.extractData)
            .catch(this.handleErrorGetSigningKeys);
    };
    /**
     * @param {?} res
     * @return {?}
     */
    OidcSecurityService.prototype.extractData = function (res) {
        var /** @type {?} */ body = res.json();
        return body;
    };
    /**
     * @param {?} error
     * @return {?}
     */
    OidcSecurityService.prototype.handleErrorGetSigningKeys = function (error) {
        var /** @type {?} */ errMsg;
        if (error instanceof Response) {
            var /** @type {?} */ body = error.json() || '';
            var /** @type {?} */ err = body.error || JSON.stringify(body);
            errMsg = error.status + " - " + (error.statusText || '') + " " + err;
        }
        else {
            errMsg = error.message ? error.message : error.toString();
        }
        console.error(errMsg);
        return Observable.throw(errMsg);
    };
    /**
     * @return {?}
     */
    OidcSecurityService.prototype.runTokenValidatation = function () {
        var _this = this;
        var /** @type {?} */ source = Observable.timer(3000, 3000)
            .timeInterval()
            .pluck('interval')
            .take(10000);
        var /** @type {?} */ subscription = source.subscribe(function () {
            if (_this.isAuthorized) {
                if (_this.oidcSecurityValidation.isTokenExpired(_this.oidcSecurityCommon.retrieve(_this.oidcSecurityCommon.storage_id_token))) {
                    _this.oidcSecurityCommon.logDebug('IsAuthorized: id_token isTokenExpired, start silent renew if active');
                    if (_this.authConfiguration.silent_renew) {
                        _this.refreshSession();
                    }
                    else {
                        _this.resetAuthorizationData();
                    }
                }
            }
        }, function (err) {
            _this.oidcSecurityCommon.logError('Error: ' + err);
        }, function () {
            _this.oidcSecurityCommon.logDebug('Completed');
        });
    };
    return OidcSecurityService;
}());
OidcSecurityService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
OidcSecurityService.ctorParameters = function () { return [
    { type: Http, },
    { type: AuthConfiguration, },
    { type: Router, },
    { type: OidcSecurityCheckSession, },
    { type: OidcSecuritySilentRenew, },
    { type: OidcSecurityUserService, },
    { type: OidcSecurityCommon, },
    { type: AuthWellKnownEndpoints, },
]; };
OidcSecurityService.propDecorators = {
    'onUserDataLoaded': [{ type: Output },],
};
var AuthModule = (function () {
    function AuthModule() {
    }
    /**
     * @return {?}
     */
    AuthModule.forRoot = function () {
        return {
            ngModule: AuthModule,
            providers: [
                OidcSecurityService,
                OidcSecurityValidation,
                OidcSecurityCheckSession,
                OidcSecuritySilentRenew,
                OidcSecurityUserService,
                OidcSecurityCommon,
                AuthConfiguration,
                AuthWellKnownEndpoints
            ]
        };
    };
    /**
     * @return {?}
     */
    AuthModule.forChild = function () {
        return {
            ngModule: AuthModule,
            providers: [
                OidcSecurityService,
                OidcSecurityValidation,
                OidcSecurityCheckSession,
                OidcSecuritySilentRenew,
                OidcSecurityUserService,
                OidcSecurityCommon,
                AuthConfiguration,
                AuthWellKnownEndpoints
            ]
        };
    };
    return AuthModule;
}());
AuthModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ]
            },] },
];
/**
 * @nocollapse
 */
AuthModule.ctorParameters = function () { return []; };
// Public classes.
/**
 * Angular OpenID Connect Implicit Flow.
 * OpenID Connect Implicit Flow RP Client
 * Written by Damien Bowden.
 * MIT license.
 * https://github.com/damienbod/angular-auth-oidc-client
 */
/**
 * Generated bundle index. Do not edit.
 */
export { OidcSecurityService, OidcSecurityValidation, OidcSecurityCheckSession, OidcSecuritySilentRenew, OidcSecurityUserService, OidcSecurityCommon, AuthWellKnownEndpoints, AuthConfiguration, AuthModule };
//# sourceMappingURL=angular-auth-oidc-client.es5.js.map
